@@ -393,8 +393,6 @@ AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Componen
 {
     const QStringList pkgNames = component.packageNames();
     Q_ASSERT(!pkgNames.isEmpty());
-    Q_ASSERT(component.kind() != AppStream::Component::KindFirmware);
-
     auto appId = makeAppId(component.id());
     auto resource = qobject_cast<AppPackageKitResource *>(m_packages.packages.value(appId));
     if (!resource) {
@@ -1059,7 +1057,7 @@ void PackageKitBackend::foundNewMajorVersion(const AppStream::Release &release)
             getUpdatesFinished(e, x);
         });
         Q_EMIT inlineMessageChanged({});
-        ResourcesModel::global()->switchToUpdates();
+        Q_EMIT ResourcesModel::global()->switchToUpdates();
     });
 
     info = i18nc("@info:status %1 is a new major version of the user's distro", "%1 is now available.", newDistroVersionText);
@@ -1072,7 +1070,7 @@ void PackageKitBackend::foundNewMajorVersion(const AppStream::Release &release)
         return;
     }
 
-    // No updates pending or avaiable. We are good to offer the upgrade to the
+    // No updates pending or available. We are good to offer the upgrade to the
     // next major version!
     Q_EMIT inlineMessageChanged(majorUpgradeAvailableMessage);
 }
