@@ -29,7 +29,7 @@ public:
     QString comment() override;
     QString name() const override;
     Q_SCRIPTABLE QString packageName() const override;
-    QStringList categories() override;
+    bool hasCategory(const QString &category) const override;
     QJsonArray licenses() override;
     QString longDescription() override;
     QList<PackageState> addonsInformation() override;
@@ -79,13 +79,16 @@ public:
     Q_SCRIPTABLE bool isPending();
 
     /* Returns true only if the deployment is from a classic Ostree format */
-    bool isClassic();
+    bool isClassic() const;
 
     /* Returns true only if the deployment is from an OCI based format */
-    bool isOCI();
+    bool isOCI() const;
+
+    /* Returns true only if the deployment is from a local OCI source */
+    bool isLocalOCI() const;
 
     /* Get the full repo:tag reference, only if it's OCI */
-    QString OCIUrl();
+    QString OCIUrl() const;
 
 private:
     QString m_name;
